﻿/************************************************************************************************************************************
 * Company Name:Mindfire Solutions.
 * Created Date:
 * Description:This is a webpart(basically a user control) that can be used to add a Subscriber or Subscriber List to MailChimp from 
 *             ExcelSheet and send email through MailChimp which is a browser based Mail service.
 * ***********************************************************************************************************************************/
using System;
using System.Collections.Generic;
using System.Linq;
using System.Web.Services;

using System.Data;
using System.Text;
using System.IO;
using System.Data.OleDb; 

using PerceptiveMCAPI;
using PerceptiveMCAPI.Types;
using PerceptiveMCAPI.Methods;
using CMS.EventLog;


/// <summary>
/// Summary description for MailChimp
/// </summary>
[WebService(Namespace = "http://tempuri.org/")]
[WebServiceBinding(ConformsTo = WsiProfiles.BasicProfile1_1)]
// To allow this Web Service to be called from script, using ASP.NET AJAX, uncomment the following line. 
[System.Web.Script.Services.ScriptService]
public class MailChimp : System.Web.Services.WebService
{

    public MailChimp()
    {

        //Uncomment the following line if using designed components 
        //InitializeComponent(); 
    }

    #region WebMethod

    /// <summary>
    /// method to verify API Key value
    /// </summary>
    /// <param name="apiKey"></param>
    /// <returns></returns>
    [WebMethod]
    public string VerifyAPIKeyVal(string apiKey)
    {
        try
        {
            StringBuilder str = new StringBuilder();

            //pingInput class takes APIKey as input parameter 
            pingInput input = new pingInput(apiKey);
            ping ping = new ping(input);
            pingOutput output = ping.Execute();//output returns "Everything's Chimpy!" if everything is chimpy, otherwise returns null

            str.Append("[{\"Result\":\"" + output.result + "\"}]");
            return str.ToString();
        }
        catch (Exception ex)
        {
            //writes the exception to the event log.
            EventLogProvider.LogException("VerifyAPIKeyVal() of MailChimp_WebPart", "Exception", ex);
            return null;
        }
    }

    /// <summary>
    /// method to returns datas to bind List and Catagory dropdown
    /// </summary>
    /// <param name="apiKey"></param>
    /// <returns></returns>
    [WebMethod]
    public string OpenMailSendPopup(string apiKey)
    {
        try
        {
            Dictionary<string, string> mailchimpList = BindList(apiKey);
            List<string> catagory = BindCatagory(apiKey);
            StringBuilder str = new StringBuilder();
            foreach (var pair in mailchimpList)
            {
                if (str.Length > 0)
                {
                    str.Append(",");
                    str.Append("{\"ListKey\":\"" + pair.Key + "\",\"ListValue\":\"" + pair.Value + "\",\"Catagory\":\"" + 1 + "\"}");
                }
                else
                {
                    str.Append("[");
                    str.Append("{\"ListKey\":\"" + pair.Key + "\",\"ListValue\":\"" + pair.Value + "\",\"Catagory\":\"" + 1 + "\"}");
                }
            }

            for (int k = 0; k < catagory.Count; k++)
            {
                if (str.Length > 0)
                {
                    str.Append(",");
                    str.Append("{\"ListKey\":\"" + catagory[k] + "\",\"ListValue\":\"" + catagory[k] + "\",\"Catagory\":\"" + 0 + "\"}");
                }
                else
                {
                    str.Append("{\"ListKey\":\"" + catagory[k] + "\",\"ListValue\":\"" + catagory[k] + "\",\"Catagory\":\"" + 0 + "\"}");
                }
            }
            str.Append("]");
            return str.ToString();
        }
        catch (Exception ex)
        {
            //writes the exception to the event log.
            EventLogProvider.LogException("OpenMailSendPopup() of MailChimp_WebPart", "Exception", ex);
            return null;
        }
    }

    /// <summary>
    /// method to retrieve datas to bind List dropdown
    /// </summary>
    /// <param name="apiKey"></param>
    /// <returns></returns>
    [WebMethod]
    private Dictionary<string, string> BindList(string apiKey)
    {
        try
        {
            Dictionary<string, string> lst = new Dictionary<string, string>();

            listsInput lstinput = new listsInput(apiKey); //listInput class takes APIKey as input parameter 
            lists list = new lists(lstinput);
            listsOutput lstoutput = list.Execute();

            for (int i = 0; i < lstoutput.result.data.Count; i++)
            {
                lst.Add(Convert.ToString(lstoutput.result.data[i].id), Convert.ToString(lstoutput.result.data[i].name));
            }
            return lst;
        }
        catch (Exception ex)
        {
            //writes the exception to the event log.
            EventLogProvider.LogException("BindList() of MailChimp_WebPart", "Exception", ex);
            return null;
        }
    }

    /// <summary>
    /// method to retrieve datas to bind Catagory dropdown
    /// </summary>
    /// <param name="apiKey"></param>
    /// <returns></returns>
    [WebMethod]
    private List<string> BindCatagory(string apiKey)
    {
        try
        {
            List<string> objList = new List<string>();

            //Add template parameters like appkey and which type of template you want to show
            templatesParms parm = new templatesParms();
            parm.apikey = apiKey;
            parm.category = "";
            parm.inactives.include = true;
            parm.inactives.only = true;
            parm.types.@base = true;
            parm.types.gallery = true;
            parm.types.user = true;

            templatesInput input = new templatesInput(parm);
            templates temp = new templates(input);

            //execute and return the template details
            templatesOutput output = temp.Execute();

            var catagory = output.result.gallery.GroupBy(p => p.category);

            foreach (var o in catagory)
            {
                objList.Add(o.Key);
            }
            return objList;
        }
        catch (Exception ex)
        {
            //writes the exception to the event log.
            EventLogProvider.LogException("BindCatagory() of MailChimp_WebPart", "Exception", ex);
            return null;
        }
    }
    
    /// <summary>
    /// method to retrieve datas to bind Template dropdown
    /// </summary>
    /// <param name="apiKey"></param>
    /// <param name="catagory"></param>
    /// <returns></returns>
    [WebMethod]
    public string BindTemplate(string apiKey, string catagory)
    {
        try
        {
            Dictionary<string, string> templateList = new Dictionary<string, string>();
            StringBuilder str = new StringBuilder();
        
            //Add template parameters like appkey and which type of template you want to show
            templatesParms parm = new templatesParms();
            parm.apikey = apiKey;
            parm.category = catagory;
            parm.inactives.include = true;
            parm.inactives.only = true;
            parm.types.@base = true;
            parm.types.gallery = true;
            parm.types.user = true;

            templatesInput input = new templatesInput(parm);
            templates temp = new templates(input);

            //execute and return the template details
            templatesOutput output = temp.Execute();

            for (int j = 0; j < output.result.gallery.Count; j++)
            {
                templateList.Add(Convert.ToString(output.result.gallery[j].id), Convert.ToString(output.result.gallery[j].name));
            }
       
            foreach (var pair in templateList)
            {
                if(str.Length > 0)
                {
                    str.Append(",");
                    str.Append("{\"Id\":\"" + pair.Key + "\",\"Name\":\"" + pair.Value + "\"}");            
                }
                else
                {
                    str.Append("[");
                    str.Append("{\"Id\":\"" + pair.Key + "\",\"Name\":\"" + pair.Value + "\"}");       
                }
            }
            str.Append("]");
            return str.ToString();
        }
        catch (Exception ex)
        {
            //writes the exception to the event log.
            EventLogProvider.LogException("BindTemplate() of MailChimp_WebPart", "Exception", ex);
            return null;
        }
    }

    /// <summary>
    /// method to check Subscriber Exist or not
    /// </summary>
    /// <param name="apiKey"></param>
    /// <param name="listId"></param>
    /// <returns></returns>
    [WebMethod]
    public string SubscriberExist(string apiKey, string listId)
    {
        try
        {
            StringBuilder str = new StringBuilder();

            //check whether Subscriber exist in List or not
            listMembersInput input = new listMembersInput(apiKey, listId, EnumValues.listMembers_status.subscribed);
            listMembers lst = new listMembers(input);
            listMembersOutput output = lst.Execute();

            str.Append("[{\"Result\":\"" + output.result.data.Count + "\"}]");
            return str.ToString();
        }
        catch (Exception ex)
        {
            //writes the exception to the event log.
            EventLogProvider.LogException("SubscriberExist() of MailChimp_WebPart", "Exception", ex);
            return null;
        }
    }

    /// <summary>
    /// method to add a subscriber to MailChimp List
    /// </summary>
    /// <param name="apiKey"></param>
    /// <param name="listId"></param>
    /// <param name="subFname"></param>
    /// <param name="subLname"></param>
    /// <param name="subEmail"></param>
    /// <returns></returns>
    [WebMethod]
    public string AddSubscriber(string apiKey, string listId, string subFname, string subLname, string subEmail)
    {
        try
        {
            StringBuilder str = new StringBuilder();
            //call the function to add Subscriber to Mailchimp list
            bool result = AddSubscribeToList(apiKey, listId, subFname, subLname, subEmail);

            if (result == true)
            {
                str.Append("[{\"Result\":\"" + result.ToString() + "\"}]");
            }
            else
            {
                str.Append("[{\"Result\":\"" + result.ToString() + "\"}]");
            }
            return str.ToString();
        }
        catch (Exception ex)
        {
            //writes the exception to the event log.
            EventLogProvider.LogException("AddSubscriber() of MailChimp_WebPart", "Exception", ex);
            return null;
        }
    }

    /// <summary>
    /// method to add SubscriberList to MailChimp List
    /// </summary>
    /// <param name="apiKey"></param>
    /// <param name="listId"></param>
    /// <returns></returns>
    [WebMethod]
    public string AddSubscriberList(string apiKey, string listId)
    {
        try
        {
            StringBuilder str = new StringBuilder();

            bool result = false;
            //target filepath to store excel file
            string target = Server.MapPath(string.Format("~/{0}/","MailChimpUploads"));

            //Initializes the new instance of DirectoryInfo class on specified path
            DirectoryInfo dir = new DirectoryInfo(target);

            //Checks if the particular directory does not exist, creates a new one
            if (dir.Exists == true)
            {
                string[] fileEntries = Directory.GetFiles(target);
                foreach (string fileName in fileEntries)
                {
                    string extension = System.IO.Path.GetExtension(fileName);
                    if (extension == ".xls" || extension == ".xlsx")
                    {
                        string connstr = string.Empty;
                        if (extension == ".xls")
                        {
                            //add connectionstring for .xls file
                            connstr = System.Configuration.ConfigurationSettings.AppSettings["ConnectionStringExcel03"].ToString();
                        }

                        else
                        {
                            //add connectionstring for .xlsx file
                            connstr = System.Configuration.ConfigurationSettings.AppSettings["ConnectionStringExcel07"].ToString();
                        }

                        connstr = String.Format(connstr, fileName);
                        OleDbConnection conn = new OleDbConnection(connstr);
                        conn.Open();//open connection
                        string strSQL = "SELECT * FROM [Sheet1$]";

                        OleDbCommand cmd = new OleDbCommand(strSQL, conn);
                        DataSet ds = new DataSet();
                        OleDbDataAdapter da = new OleDbDataAdapter(cmd);
                        da.Fill(ds);

                        DataTable dt = ds.Tables[0];
                        //loop to add all the Subscriber to MailChimp List 
                        for (int i = 0; i < dt.Rows.Count; i++)
                        {
                            string subEmail = dt.Rows[i][0].ToString();
                            string subFname = dt.Rows[i][1].ToString();
                            string subLname = dt.Rows[i][2].ToString();
                            //call the function to add Subscriber to Mailchimp list
                            result = AddSubscribeToList(apiKey, listId, subFname, subLname, subEmail); 
                        }
						
						conn.Close();
                    }
                }
            }

            if (result == true)
            {
                str.Append("[{\"Result\":\"" + result.ToString() + "\"}]");
            }
            else
            {
                str.Append("[{\"Result\":\"" + result.ToString() + "\"}]");
            }
            return str.ToString();
        }
        catch (Exception ex)
        {
            //writes the exception to the event log.
            EventLogProvider.LogException("AddSubscriberList() of MailChimp_WebPart", "Exception", ex);
            return null;
        }
    }

    /// <summary>
    /// method to add Subscriber 
    /// </summary>
    /// <param name="apiKey"></param>
    /// <param name="listId"></param>
    /// <param name="Fname"></param>
    /// <param name="Lname"></param>
    /// <param name="Email"></param>
    /// <returns></returns>
    [WebMethod]
    private bool AddSubscribeToList(string apiKey, string listId, string Fname, string Lname, string Email)
    {
        try
        {
            listSubscribe list = new listSubscribe();

            //create a listSubscribeInput to add details to MailChimp list
            listSubscribeInput input = new listSubscribeInput();

            //directive overrides
            input.api_AccessType = PerceptiveMCAPI.EnumValues.AccessType.Serial;
            input.api_OutputType = EnumValues.OutputType.JSON;
            input.api_CustomErrorMessages = true;
            input.api_MethodType = PerceptiveMCAPI.EnumValues.MethodType.POST;
            input.api_Validate = true;
            input.api_OutputType = PerceptiveMCAPI.EnumValues.OutputType.XML;

            //method parameters
            input.parms.email_address = Email.ToString();
            input.parms.send_welcome = true;
            input.parms.update_existing = false;
            input.parms.replace_interests = true;
            input.parms.double_optin = false;
            input.parms.merge_vars.Add("FNAME", Fname.ToString());
            input.parms.merge_vars.Add("LNAME", Lname.ToString());
            input.parms.apikey = apiKey;
            input.parms.id = listId.ToString();

            //execute SubscriberList and add it to Mailchimp list
            listSubscribeOutput output = list.Execute(input);
            return output.result;
        }
        catch (Exception ex)
        {
            //writes the exception to the event log.
            EventLogProvider.LogException("AddSubscribeToList() of MailChimp_WebPart", "Exception", ex);
            return false;
        }
    }

    #endregion

}
